#include <QTest>
#if PATCHED == 1
#include "qbytedata_p.h"
#else
#include "qbytedata_p.original.h"
#endif

#define QIODEVICE_BUFFERSIZE Q_INT64_C(16384)

static QByteArray makeByteArray(int size)
{
    QByteArray array;
    array.resize(size);
    char *data = array.data();
    for (int i = 0; i < size; ++i)
	data[i] = i;
    return array;
}

class tst_QByteDataBuffer : public QObject
{
    Q_OBJECT
private Q_SLOTS:
    void canReadLine()
    {
	QByteDataBuffer buf;
	buf.append(QByteArray("a"));
	buf.append(QByteArray("\nb"));
	QVERIFY(buf.canReadLine());
	QVERIFY(buf.getChar() == 'a');
	QVERIFY(buf.canReadLine());
	QVERIFY(buf.getChar() == '\n');
	QVERIFY(!buf.canReadLine()); // skip firstPos
    }

    void positionHandling()
    {
	QByteDataBuffer buf;
	buf.append(QByteArray("abc"));
	buf.append(QByteArray("def"));

	QCOMPARE(buf.byteAmount(), 6LL);
	QCOMPARE(buf.sizeNextBlock(), 3LL);

	QCOMPARE(buf.getChar(), 'a');
	QCOMPARE(buf.byteAmount(), 5LL);
	QCOMPARE(buf.sizeNextBlock(), 2LL);

	QVERIFY(!strcmp(buf[0].constData(), "bc"));
	QCOMPARE(buf.getChar(), 'b');
	QCOMPARE(buf.byteAmount(), 4LL);
	QCOMPARE(buf.sizeNextBlock(), 1LL);

	buf.prepend(QByteArray("ab"));
	QCOMPARE(buf.byteAmount(), 6LL);
	QVERIFY(!strcmp(buf.readAll().constData(), "abcdef"));
	QCOMPARE(buf.byteAmount(), 0LL);
    }

    void appendBuffer()
    {
	QByteDataBuffer buf;
	buf.append(QByteArray("abc"));
	buf.getChar(); // increase firstPos

	QByteDataBuffer tmp;
	tmp.append(buf);
	QCOMPARE(tmp.readAll(), buf.readAll());
    }

    void fullRead()
    {
	const int arraySize = 10;
	QByteArray data = makeByteArray(arraySize);
	QByteDataBuffer buf;

	QBENCHMARK {
	    buf.clear();
	    buf.append(data);
	    buf.append(data);
	    QByteArray tmp = buf.read();
	}

	QByteArray tmp = buf.read();
	QVERIFY(data.size() == tmp.size());
	QVERIFY(!memcmp(tmp.constData(), data.constData(), arraySize));
    }

    void partialRead()
    {
	const int arraySize = 1e6;
	const int blockSize = QIODEVICE_BUFFERSIZE;
	QByteArray data = makeByteArray(arraySize);
	QByteArray tmp;
	tmp.resize(blockSize);
	QByteDataBuffer buf;

	QBENCHMARK {
	    buf.clear();
	    buf.append(data);
	    /*qint64 len = */buf.read(tmp.data(), blockSize);
	}

	for (int i = 1; i < arraySize / blockSize; ++i) {
	    qint64 len = buf.read(tmp.data(), blockSize);
	    QVERIFY(!memcmp(tmp.constData(), data.constData() + blockSize * i, len));
	}
    }
};

QTEST_MAIN(tst_QByteDataBuffer)
#include "tst_qbytedatabuffer.moc"
